@@ -69,6 +69,7 @@ struct _GVfsUDisks2Volume
 
   /* set in update_volume() */
   GIcon *icon;
+  GIcon *symbolic_icon;
   GFile *activation_root;
   gchar *name;
   gchar *sort_key;
@@ -125,10 +126,9 @@ gvfs_udisks2_volume_finalize (GObject *object)
   if (volume->mount_point != NULL)
     g_unix_mount_point_free (volume->mount_point);
 
-  if (volume->icon != NULL)
-    g_object_unref (volume->icon);
-  if (volume->activation_root != NULL)
-    g_object_unref (volume->activation_root);
+  g_clear_object (&volume->icon);
+  g_clear_object (&volume->symbolic_icon);
+  g_clear_object (&volume->activation_root);
 
   g_free (volume->name);
   g_free (volume->sort_key);
@@ -165,6 +165,7 @@ apply_options_from_fstab (GVfsUDisks2Volume *volume,
 {
   gchar *name;
   gchar *icon_name;
+  gchar *symbolic_icon_name;
 
   name = gvfs_udisks2_utils_lookup_fstab_options_value (fstab_options, "x-gvfs-name=");
   if (name != NULL)
@@ -176,14 +177,34 @@ apply_options_from_fstab (GVfsUDisks2Volume *volume,
   icon_name = gvfs_udisks2_utils_lookup_fstab_options_value (fstab_options, "x-gvfs-icon=");
   if (icon_name != NULL)
     {
+      g_clear_object (&volume->icon);
       volume->icon = g_themed_icon_new_with_default_fallbacks (icon_name);
       g_free (icon_name);
     }
+
+  symbolic_icon_name = gvfs_udisks2_utils_lookup_fstab_options_value (fstab_options, "x-gvfs-symbolic-icon=");
+  if (symbolic_icon_name != NULL)
+    {
+      g_clear_object (&volume->symbolic_icon);
+      volume->symbolic_icon = g_themed_icon_new_with_default_fallbacks (symbolic_icon_name);
+      g_free (symbolic_icon_name);
+    }
+}
+
+
+static gpointer
+_g_object_ref0 (gpointer object)
+{
+  if (object != NULL)
+    return g_object_ref (G_OBJECT (object));
+  else
+    return NULL;
 }
 
 static gboolean
 update_volume (GVfsUDisks2Volume *volume)
 {
+  UDisksClient *udisks_client;
   gboolean changed;
   gboolean old_can_mount;
   gboolean old_should_automount;
@@ -191,9 +212,12 @@ update_volume (GVfsUDisks2Volume *volume)
   gchar *old_device_file;
   dev_t old_dev;
   GIcon *old_icon;
+  GIcon *old_symbolic_icon;
   UDisksDrive *udisks_drive;
   gchar *s;
 
+  udisks_client = gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor);
+
   /* ---------------------------------------------------------------------------------------------------- */
   /* save old values */
 
@@ -203,6 +227,7 @@ update_volume (GVfsUDisks2Volume *volume)
   old_device_file = g_strdup (volume->device_file);
   old_dev = volume->dev;
   old_icon = volume->icon != NULL ? g_object_ref (volume->icon) : NULL;
+  old_symbolic_icon = volume->symbolic_icon != NULL ? g_object_ref (volume->symbolic_icon) : NULL;
 
   /* ---------------------------------------------------------------------------------------------------- */
   /* reset */
@@ -212,6 +237,7 @@ update_volume (GVfsUDisks2Volume *volume)
   g_free (volume->device_file); volume->device_file = NULL;
   volume->dev = 0;
   g_clear_object (&volume->icon);
+  g_clear_object (&volume->symbolic_icon);
 
   /* ---------------------------------------------------------------------------------------------------- */
   /* in with the new */
@@ -226,12 +252,11 @@ update_volume (GVfsUDisks2Volume *volume)
       GVariant *configuration_value;
       UDisksLoop *loop = NULL;
 
-      loop = udisks_client_get_loop_for_block (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
+      loop = udisks_client_get_loop_for_block (udisks_client,
                                                volume->block);
 
       /* If unlocked, use the values from the unlocked block device for presentation */
-      cleartext_block = udisks_client_get_cleartext_block (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
-                                                           volume->block);
+      cleartext_block = udisks_client_get_cleartext_block (udisks_client, volume->block);
       if (cleartext_block != NULL)
         block = cleartext_block;
       else
@@ -246,8 +271,7 @@ update_volume (GVfsUDisks2Volume *volume)
         }
       else if (g_strcmp0 (udisks_block_get_id_type (block), "crypto_LUKS") == 0)
         {
-          s = udisks_client_get_size_for_display (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
-                                                  udisks_block_get_size (volume->block), FALSE, FALSE);
+          s = udisks_client_get_size_for_display (udisks_client, udisks_block_get_size (volume->block), FALSE, FALSE);
           /* Translators: This is used for encrypted volumes.
            *              The first %s is the formatted size (e.g. "42.0 MB").
            */
@@ -259,8 +283,7 @@ update_volume (GVfsUDisks2Volume *volume)
           guint64 size = udisks_block_get_size (block);
           if (size > 0)
             {
-              s = udisks_client_get_size_for_display (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
-                                                      size, FALSE, FALSE);
+              s = udisks_client_get_size_for_display (udisks_client, size, FALSE, FALSE);
               /* Translators: This is used for volume with no filesystem label.
                *              The first %s is the formatted size (e.g. "42.0 MB").
                */
@@ -269,22 +292,44 @@ update_volume (GVfsUDisks2Volume *volume)
             }
         }
 
-      udisks_drive = udisks_client_get_drive_for_block (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
-                                                        volume->block);
+      udisks_drive = udisks_client_get_drive_for_block (udisks_client, volume->block);
       if (udisks_drive != NULL)
         {
-          gchar *drive_desc;
-          GIcon *drive_icon;
-          gchar *media_desc;
-          GIcon *media_icon;
+          gchar *drive_desc = NULL;
+          GIcon *drive_icon = NULL;
+          GIcon *drive_symbolic_icon = NULL;
+          gchar *media_desc = NULL;
+          GIcon *media_icon = NULL;
+          GIcon *media_symbolic_icon = NULL;
 
-          udisks_client_get_drive_info (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
+#if UDISKS_CHECK_VERSION(2,0,90)
+          {
+            UDisksObject *object = (UDisksObject *) g_dbus_interface_get_object (G_DBUS_INTERFACE (udisks_drive));
+            if (object != NULL)
+              {
+                UDisksObjectInfo *info = udisks_client_get_object_info (udisks_client, object);
+                if (info != NULL)
+                  {
+                    drive_desc = g_strdup (udisks_object_info_get_description (info));
+                    drive_icon = _g_object_ref0 (udisks_object_info_get_icon (info));
+                    drive_symbolic_icon = _g_object_ref0 (udisks_object_info_get_icon_symbolic (info));
+                    media_desc = g_strdup (udisks_object_info_get_media_description (info));
+                    media_icon = _g_object_ref0 (udisks_object_info_get_media_icon (info));
+                    media_symbolic_icon = _g_object_ref0 (udisks_object_info_get_media_icon_symbolic (info));
+                    g_object_unref (info);
+                  }
+              }
+          }
+#else
+          udisks_client_get_drive_info (udisks_client,
                                         udisks_drive,
                                         NULL, /* drive_name */
                                         &drive_desc,
                                         &drive_icon,
                                         &media_desc,
                                         &media_icon);
+#endif
+
           if (media_desc == NULL)
             {
               media_desc = drive_desc;
@@ -295,6 +340,11 @@ update_volume (GVfsUDisks2Volume *volume)
               media_icon = drive_icon;
               drive_icon = NULL;
             }
+          if (media_symbolic_icon == NULL)
+            {
+              media_symbolic_icon = drive_symbolic_icon;
+              drive_symbolic_icon = NULL;
+            }
 
           /* Override name for blank and audio discs */
           if (udisks_drive_get_optical_blank (udisks_drive))
@@ -309,6 +359,7 @@ update_volume (GVfsUDisks2Volume *volume)
             }
 
           volume->icon = media_icon != NULL ? g_object_ref (media_icon) : NULL;
+          volume->symbolic_icon = media_symbolic_icon != NULL ? g_object_ref (media_symbolic_icon) : NULL;
 
           /* use media_desc if we haven't figured out a name yet (applies to e.g.
            * /dev/fd0 since its size is 0)
@@ -320,8 +371,8 @@ update_volume (GVfsUDisks2Volume *volume)
             }
 
           g_free (media_desc);
-          if (media_icon != NULL)
-            g_object_unref (media_icon);
+          g_clear_object (&media_icon);
+          g_clear_object (&media_symbolic_icon);
 
           /* Only automount drives attached to the same seat as we're running on
            */
@@ -358,6 +409,27 @@ update_volume (GVfsUDisks2Volume *volume)
             }
           g_object_unref (udisks_drive);
         }
+      else
+        {
+          /* No UDisksDrive, but we do have a UDisksBlock (example: /dev/loop0). Use
+           * that to get the icons via UDisksObjectInfo
+           */
+#if UDISKS_CHECK_VERSION(2,0,90)
+          {
+            UDisksObject *object = (UDisksObject *) g_dbus_interface_get_object (G_DBUS_INTERFACE (volume->block));
+            if (object != NULL)
+              {
+                UDisksObjectInfo *info = udisks_client_get_object_info (udisks_client, object);
+                if (info != NULL)
+                  {
+                    volume->icon = _g_object_ref0 (udisks_object_info_get_icon (info));
+                    volume->symbolic_icon = _g_object_ref0 (udisks_object_info_get_icon_symbolic (info));
+                    g_object_unref (info);
+                  }
+              }
+          }
+#endif
+        }
 
       /* Also automount loop devices set up by the user himself - e.g. via the
        * udisks interfaces or the gnome-disk-image-mounter(1) command
@@ -383,8 +455,16 @@ update_volume (GVfsUDisks2Volume *volume)
           g_clear_object (&volume->icon);
           volume->icon = g_themed_icon_new_with_default_fallbacks (hint);
         }
+#if UDISKS_CHECK_VERSION(2,0,90)
+      hint = udisks_block_get_hint_symbolic_icon_name (volume->block);
+      if (hint != NULL && strlen (hint) > 0)
+        {
+          g_clear_object (&volume->symbolic_icon);
+          volume->symbolic_icon = g_themed_icon_new_with_default_fallbacks (hint);
+        }
+#endif
 
-      /* Use x-gvfs-name=The%20Name and x-gvfs-icon=foo-name, if available */
+      /* Use x-gvfs-name=The%20Name, x-gvfs-icon=foo-name, x-gvfs-icon=foo-name-symbolic, if available */
       g_variant_iter_init (&iter, udisks_block_get_configuration (block));
       while (g_variant_iter_next (&iter, "(&s@a{sv})", &configuration_type, &configuration_value))
         {
@@ -430,6 +510,8 @@ update_volume (GVfsUDisks2Volume *volume)
         volume->name = g_unix_mount_point_guess_name (volume->mount_point);
       if (volume->icon == NULL)
         volume->icon = gvfs_udisks2_utils_icon_from_fs_type (g_unix_mount_point_get_fs_type (volume->mount_point));
+      if (volume->symbolic_icon == NULL)
+        volume->symbolic_icon = gvfs_udisks2_utils_symbolic_icon_from_fs_type (g_unix_mount_point_get_fs_type (volume->mount_point));
     }
 
   if (volume->mount == NULL)
@@ -445,6 +527,8 @@ update_volume (GVfsUDisks2Volume *volume)
     }
   if (volume->icon == NULL)
     volume->icon = g_themed_icon_new ("drive-removable-media");
+  if (volume->symbolic_icon == NULL)
+    volume->symbolic_icon = g_themed_icon_new ("drive-removable-media-symbolic");
 
   /* ---------------------------------------------------------------------------------------------------- */
   /* compute whether something changed */
@@ -631,6 +715,13 @@ gvfs_udisks2_volume_get_icon (GVolume *_volume)
   return volume->icon != NULL ? g_object_ref (volume->icon) : NULL;
 }
 
+static GIcon *
+gvfs_udisks2_volume_get_symbolic_icon (GVolume *_volume)
+{
+  GVfsUDisks2Volume *volume = GVFS_UDISKS2_VOLUME (_volume);
+  return volume->symbolic_icon != NULL ? g_object_ref (volume->symbolic_icon) : NULL;
+}
+
 static char *
 gvfs_udisks2_volume_get_name (GVolume *_volume)
 {
@@ -1457,6 +1548,23 @@ gvfs_udisks2_volume_mount (GVolume             *_volume,
             {
               gchar *drive_name;
               gchar *drive_desc;
+
+#if UDISKS_CHECK_VERSION(2,0,90)
+              {
+                UDisksObject *object = (UDisksObject *) g_dbus_interface_get_object (G_DBUS_INTERFACE (udisks_drive));
+                if (object != NULL)
+                  {
+                    UDisksObjectInfo *info = udisks_client_get_object_info (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
+                                                                            object);
+                    if (info != NULL)
+                      {
+                        drive_name = g_strdup (udisks_object_info_get_name (info));
+                        drive_desc = g_strdup (udisks_object_info_get_description (info));
+                        g_object_unref (info);
+                      }
+                  }
+              }
+#else
               udisks_client_get_drive_info (gvfs_udisks2_volume_monitor_get_udisks_client (volume->monitor),
                                             udisks_drive,
                                             &drive_name,
@@ -1464,6 +1572,7 @@ gvfs_udisks2_volume_mount (GVolume             *_volume,
                                             NULL,  /* drive_icon */
                                             NULL,  /* media_desc */
                                             NULL); /* media_icon */
+#endif
               /* Translators: this is used to describe the drive the encrypted media
                * is on - the first %s is the name (such as 'WD 2500JB External'), the
                * second %s is the description ('250 GB Hard Disk').
@@ -1623,6 +1732,7 @@ gvfs_udisks2_volume_volume_iface_init (GVolumeIface *iface)
 {
   iface->get_name = gvfs_udisks2_volume_get_name;
   iface->get_icon = gvfs_udisks2_volume_get_icon;
+  iface->get_symbolic_icon = gvfs_udisks2_volume_get_symbolic_icon;
   iface->get_uuid = gvfs_udisks2_volume_get_uuid;
   iface->get_drive = gvfs_udisks2_volume_get_drive;
   iface->get_mount = gvfs_udisks2_volume_get_mount;
